package utils

import (
	"errors"
	"regexp"
	"strconv"

	"github.com/go-xorm/xorm"
)

// GetPageList 获取列表
func GetPageList(db *xorm.Session, target interface{}, limit []int, sqlorArgs ...interface{}) (int64, error) {
	if sqlorArgs == nil || len(sqlorArgs) == 0 {
		return 0, errors.New("SQL 语句不存在")
	}

	countSQLorArgs := GetCountSQL(sqlorArgs...)
	cntRes, err := db.Query(countSQLorArgs...)
	if err != nil {
		return 0, err
	}

	cntStr := string(cntRes[0]["cnt"])
	cnt, _ := strconv.ParseInt(cntStr, 10, 64)

	sql, args := PackLimitToSQL(limit, sqlorArgs...)

	err = db.SQL(sql, args...).Find(target)
	if err != nil {
		return 0, err
	}

	return cnt, nil
}

// GetCountSQL 获取计算 Count 的相关 SQL 脚本
func GetCountSQL(sqlorArgs ...interface{}) []interface{} {
	sql := sqlorArgs[0].(string)

	re := regexp.MustCompile("(?ims)(select .* from)")
	cntSQL := re.ReplaceAllString(sql, "select count(*) as cnt from")

	newSQLorArgs := []interface{}{cntSQL}
	newSQLorArgs = append(newSQLorArgs, sqlorArgs[1:])
	return newSQLorArgs
}

// PackLimitToSQL 添加 limit 语句到 sql 最后
func PackLimitToSQL(limit []int, sqlorArgs ...interface{}) (string, []interface{}) {
	sql := sqlorArgs[0].(string)
	args := sqlorArgs[1:]

	switch len(limit) {
	case 1:
		return sql + " LIMIT " + strconv.Itoa(limit[0]), args
	case 2:
		return sql + " LIMIT " + strconv.Itoa(limit[0]) + " OFFSET " + strconv.Itoa(limit[1]), args
	default:
		return sql, args
	}
}
