package utils_test

import (
	"spaceofcheng/services/utils"
	"testing"
)

func TestGetCountSQL(t *testing.T) {
	testCases := []map[string]string{
		map[string]string{
			"sql":      "Select field1, field2 From table",
			"expected": "select count(*) as cnt from table",
		},
		map[string]string{
			"sql": `Select 
								field1, field2 From table where status = ?`,
			"expected": "select count(*) as cnt from table where status = ?",
		},
	}

	for inx, cs := range testCases {
		sql := cs["sql"]
		expected := cs["expected"]

		var res []interface{}
		if inx < 1 {
			res = utils.GetCountSQL(sql)
		} else {
			res = utils.GetCountSQL(sql, inx)
		}

		if res == nil || len(res) == 0 {
			t.Fatalf("Test GetCountSQL fail --1")
		}

		newSQL := res[0].(string)
		if newSQL != expected {
			t.Fatalf("Test GetCountSQL fail: %s", newSQL)
		}
	}
}

func TestPackLimitToSQL(t *testing.T) {
	testCases := []map[string]string{
		map[string]string{
			"sql":      "Select * From table",
			"expected": "Select * From table LIMIT 1 OFFSET 0",
		},
		map[string]string{
			"sql":      "Select * From table",
			"expected": "Select * From table LIMIT 1",
		},
	}

	for inx, cs := range testCases {
		sql := cs["sql"]
		expected := cs["expected"]

		var sqlRes string
		if inx < 1 {
			sqlRes, _ = utils.PackLimitToSQL([]int{inx + 1, inx}, sql)
		} else {
			sqlRes, _ = utils.PackLimitToSQL([]int{inx}, sql, inx)
		}

		if sqlRes != expected {
			t.Fatalf("\n%v: Test PackLimitToSQL fail: %s", inx+1, sqlRes)
		}
	}

}
